#include "jpg2pcx.h"

int pcxwrite(char * pcxname, unsigned char * buf, int xsize, int ysize)
{
int fd,n,col,pix,x,y;
int cntptr,xbuf,yptr,fsize;
unsigned char pcxhead[128];
unsigned char pcxline[4096];
unsigned char pcxpal[768]; // effektiv nur 3*192 Farben gut komprimierbar

if (xsize>4095) { fprintf(stderr,"image too big\n"); return 1; };

fd = open(pcxname, O_WRONLY|O_CREAT|O_EXCL|0400000, 0644); 
  // 04... ist O_NOFOLLOW
  // O_EXCL: do not open if file already exists
if (!fd) { fprintf(stderr,"could not write PCX image\n"); return 1; };
fsize=0;

for (n=0;n<128;n++) pcxhead[n]=0;
#define w2head(a,n) { pcxhead[n]=((a) & 255); pcxhead[(n)+1]=((a) >> 8); }
#define pal2head(r,g,b,n) { pcxhead[16+(n)+(n)+(n)]=r; \
    pcxhead[17+(n)+(n)+(n)]=g; pcxhead[18+(n)+(n)+(n)]=b; }

/* 0000000: 0a05 0108 0000 0000 5702 4f03 4800 4800 */
pcxhead[0]=10; pcxhead[1]=5; pcxhead[2]=1; pcxhead[3]=8; 
/* magic, version, planes, bpp */
w2head(0,4); w2head(0,6); w2head(xsize-1,8); w2head(ysize-1,10); /* ecken */
w2head(72,12); w2head(72,14); /* dpi */

/* 0000010: sechzehn mal drei Bytes (r, g, b)       */
// pal2head(  0,  0,  0, 0);  pal2head(  0,  0,255, 1); 
// pal2head(  0,255,  0, 2);  pal2head(  0,255,255, 3);
// pal2head(255,  0,  0, 4);  pal2head(255,  0,255, 5); 
// pal2head(255,255,  0, 6);  pal2head(255,255,255, 7);
// pal2head(  0,  0,  0, 8);  pal2head(  0,  0,127, 9); 
// pal2head(  0,127,  0,10);  pal2head(  0,127,127,11);
// pal2head(127,  0,  0,12);  pal2head(127,  0,127,13); 
// pal2head(127,127,  0,14);  pal2head(127,127,127,15);
/* evtl. mehr Graustufen, kein CMY / nur RGB, Farbe + W ausser Farbe + S */

palcreate(&pcxpal[0]);
for (n=0;n<16;n++) pal2head(pcxpal[n+n+n],pcxpal[n+n+n+1],pcxpal[n+n+n+2],n);

/* 0000040: 0001 5802 0100 00 ... 00 Head=128 Bytes */
pcxhead[64]=0; pcxhead[65]=1; 
w2head(xsize,66); /* -, planes, linebytes */
w2head(1,68); /* palinfo */
write(fd,&pcxhead[0],128); /* write pcx header */
	fsize+=128;

for (y=0;y<ysize;y++)
  { col=42; n=0; xbuf=0; pix=0; cntptr=-1;
    yptr=((ysize-1)-y) * xsize * 3; // tga: top first / pcx: bottom first!
    for (x=0;x<xsize;x++)
    { pix = buf[yptr+x+x+x];  // TAKE COLOR INDEX FROM EX-R-PLANE
      if (pix==col)
        { // start line with 2 pixels min
          if (cntptr<0) 
            { cntptr=xbuf-1;
              pcxline[cntptr]=0xc1; // replace pixel with line header
              pcxline[xbuf]=col;    // write line color
              xbuf++;
            };
          // make line longer
          if (pcxline[cntptr]<255)  // max 0xc0 | 0x3f is ok
            { pcxline[cntptr]++; }  // increase line counter
          else
            { // yuck, counter overrun - so pretend its a normal pixel!
              cntptr=-1;
              pcxline[xbuf]=col;
              xbuf++;
            };
        }
      else
        { // line has ended, start with single pixel
          cntptr=-1;
          col=pcxline[xbuf]=pix;    // write standard pixel
          xbuf++;
        };
    }; // end of for x loop
    
    if ((n=write(fd,&pcxline[0],xbuf))<xbuf)
      { fprintf(stderr,"write error in line %d\n",y); return 1; 
      };
    fsize+=xbuf;
  }; // end of for y loop

  for (n=64;n<128;n++) pcxhead[n]=0; // prepare some 0 bytes

  pcxhead[0]=12;                     // magic value for colormap
  if (write(fd,&pcxhead[0],1)!=1)
    { fprintf(stderr,"write error while writing pal256\n"); 
      return 1;
    };
  fsize++;

  // write 256 colors because we use 8bpp (!)
  if ((n=write(fd,&pcxpal[0],768))<768)
        { fprintf(stderr,"write error while writing pal256\n"); 
          return 1;
        };
  fsize+=768;

close(fd);
fprintf(stderr,"Saved %dx%d PCX image to %d bytes PCX file %s\n",
               xsize,ysize,fsize,pcxname);
return 0;
};

// --------------------------------------------------------------------

int rufe_convert(char * rein, char * raus,char * opt1, char * opt2, char * opt3)
{ 
  int pid,status,n;
  fprintf(stderr,"convert %s %s %s %s %s\n",opt1,opt2,opt3,rein,raus);
  pid = fork();
  if (pid == -1) return -1;
  if (pid == 0) 
  {
    char *argv2[7];
    char *env2[7];
    argv2[0] = "convert"; // mit execve statt ... execvp ganzer Pfad noetig
    n=1;
    if (opt1) { argv2[n]=opt1; n++; };
    if (opt2) { argv2[n]=opt2; n++; };
    if (opt3) { argv2[n]=opt3; n++; };
    if (rein) { argv2[n]=rein; n++; };
    if (raus) { argv2[n]=raus; n++; };
    argv2[n]=NULL;

    env2[0] = malloc(5+sizeof(_PATH_TMP));
    env2[0][4+sizeof(_PATH_TMP)] = '\0';
    snprintf(env2[0],4+sizeof(_PATH_TMP),"PWD=%s",_PATH_TMP);

    env2[1] = "SHELL=/sbin/DENY"; // oder _PATH_BSHELL
    env2[2] = getenv("HOME"); env2[3] = getenv("PWD"); 

    /* env2[??] = getenv("USER"); env2[??] = getenv("LOGNAME"); */
    env2[4] = "IFS=\n";       env2[5] = "LANG=de_DE"; 
    env2[6] = NULL;          // sinnvolle envvars: 
    // PATH MANPATH USER/LOGNAME HOME EDITOR/VISUAL
    // IFS LANG/LC_ALL PWD PPID SHELL... (getenv liefert evtl. NULL)

    execve("/usr/X11R6/bin/convert", argv2, env2);
    // execve ruft kernelcall, execvp kopiert env
    fprintf(stderr,"execve failed\n");
    exit(127); // never reached
  }
  for (;;) 
  {
    if (waitpid(pid, &status, 0) == -1) 
    {
      if (errno != EINTR) 
      { fprintf(stderr,"EINTR while waiting for child\n");
        return -1;
      };
    } else return status;
  };
};
